#!/usr/bin/perl

# Copyright (C) 2001.2002 Karsten Reincke <karsten.reincke@karubik.de>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#  
# file <merge-mrproject-vtodos.pl> version <#1.3.1#> of project <MrProjext>

#usage();

$number=$#ARGV+1;

if (($#ARGV+1)!=3)
{
  usage();
}

$argNumber=0;
$mrProjextFile="";
$mergeInFile="";
$mergeInFileType="";

while (defined($ca=shift))
{
  if ($argNumber ==0)
  {
    $mrProjextFile=$ca;
  }
  elsif ($argNumber ==1)
  {
    $mergeInFileType=$ca;
  }
  elsif ($argNumber ==2)
  {
    $mergeInFile=$ca;
  }
  else
  {
    usage();
  }
  
  if (($ca eq "-h")||($ca eq "--help"))
  {
    usage();
  }
  $argNumber+=1;
}

if ($argNumber!=3)
{
    usage();
}

$projectName=findProjectName($mrProjextFile);
if ($projectName eq "")
{
  print(STDERR "$mrProjextFile has not been generated by MrProjext with --rwVcardFile\n");
  usage();
}

if (!(open GMPSOURCE,"< ${mrProjextFile}"))
{
  print(STDERR "can't open ${mrProjextFile} for reading\n");
  usage();
}

if (!(open GVCSOURCE, "< ${mergeInFile}"))
{
  print(STDERR "can't open ${mergeInFile} for reading\n");
  usage();
}

rewriteMergeInFileWithoutMrProjectTasksAndClosingTag($projectName);
rewriteMrProjextFileWithoutOpeningAndClosingTags();

print(STDOUT "\nEND:VCALENDAR\n");

close(GMPSOURCE);
close(GVCSOURCE);

exit(0);


sub usage
{

  print( STDERR "usage is ...\n");
  print( STDERR "merge-mrproject-vtodos.pl mrprojext-vcard-file [-gf|-ef] vcard-file\n");
  print( STDERR "where ...\n");
  print( STDERR "a) mrprojext-vcard-file :- the mrpojext-extraction (--rwVcardFile)\n");
  print( STDERR "b.1) -gf :- merge into a gnomecal-vcard-file\n");
  print( STDERR "b.2) -ef :- merge into an evolution-vcard-file\n");
  print( STDERR "c) vcard-file :- path to the gnomecal- (evolution-) vcard-file\n");
  exit(0);
}


sub findProjectName
{
  local $mrProjextFile=shift;
  
  if (!(open(LMPF,"< ${mrProjextFile}")))
  {
    print(STDERR "can't open ${mrProjextFile}\n");
    usage();
  }
  while($line=<LMPF>)
  {
    if ($line=~ m/^UID:mrproject-(.*)-/)
    {
      return $1;
    }
  }
  close(LMPF);
  return "";
}

sub rewriteMergeInFileWithoutMrProjectTasksAndClosingTag
{
  local $lProjectName=shift;
  local $lVtodoOpen=0;
  local $writeLine=1;
  local $line;
  $/="\n";
  while(defined($line=<GVCSOURCE>))
  {   
    chomp $line;
    
    if ($line=~ m/END:VCALENDAR/)
    {
      return 1;
    }
    
    if ($line=~ m/BEGIN\:VTODO/)
    {
      $lVtodoOpen=1;
      $writeLine=0;
      next
    } 
    
    if ($line=~ m/END\:VTODO/)
    {
      $lVtodoOpen=0;
      if ($writeLine)
      {
      	print(STDOUT "$line\n");
      }
      else
      {
      	$writeLine=1;
      }
      next;
    } 
    
    if(($lVtodoOpen)&&($line=~ m/UID\:/))
    {
      if (!($line=~ m/mrproject-${lProjectName}/))
      {
      	print(STDOUT "BEGIN:VTODO\n");
	$writeLine=1;	
      }
    }
    
    if ($writeLine) 
    {
      print(STDOUT "$line\n");
    }
  }
  return 1;
}

sub rewriteMrProjextFileWithoutOpeningAndClosingTags
{
  local $secondline;
  $/="\n";
  while(defined($secondline=<GMPSOURCE>))
  {   
    chomp $secondline;

    if ($secondline=~ m/END\:VCALENDAR/)
    {
      return 1;
    }
    if ($secondline=~ m/BEGIN\:VCALENDAR/)
    {
      next;
    }
    print(STDOUT "$secondline\n");
  }
  return 1;
}
